@@ -2,10 +2,11 @@
 
 import sys
 from enum import Enum
-from collections import ChainMap
 from typing import Any, Callable, Iterator, Dict, Optional, Sequence, TextIO, Union
 from .input import get_char
-from .keys import ACTIONS, KEYS
+from .keys import get_keycodes_actions
+from .exceptions import AcceptAction, AbortAction, CustomAction
+from .line_editor import LineEditor
 from .ansi import (  # noqa
     NL,
     SPACE,
@@ -26,7 +27,7 @@ from .ansi import (  # noqa
     PURPLE_BG,
     CYAN_BG,
     WHITE_BG,
-    NORMAL,
+    RESET,
     BOLD,
     NEGATIVE,
 )
@@ -34,24 +35,20 @@ from .screen import Screen
 
 __all__ = [
     "Finder",
+    "CustomAction",
     "Layout",
     "InfoStyle",
     "DEFAULT_POINTER",
     "DEFAULT_PROMPT",
+    "DEFAULT_HEADER",
 ]
 
 DEFAULT_POINTER = ">"
 "Default pointer"
 DEFAULT_PROMPT = ">"
 "Default input prompt"
-
-
-class Confirm(Exception):
-    pass
-
-
-class Cancel(Exception):
-    pass
+DEFAULT_HEADER = ""
+"Default header"
 
 
 class Layout(Enum):
@@ -81,6 +78,8 @@ class Finder:
         info_style: InfoStyle = InfoStyle.DEFAULT,
         pointer_str: str = DEFAULT_POINTER,
         prompt_str: str = DEFAULT_PROMPT,
+        header_str: str = DEFAULT_HEADER,
+        actions: Optional[Dict[str, Sequence[str]]] = None,
         output_stream: TextIO = sys.stderr,
     ):
         """
@@ -95,6 +94,9 @@ class Finder:
             info_style: Determines the display style of finder info
             pointer_str: Pointer to the current line
             prompt_str: Input prompt
+            header_str: Header
+            actions: Custom key binding
+            output_stream: Output stream
         """
         self.fullscreen = fullscreen
         self.height = height
@@ -104,8 +106,9 @@ class Finder:
         self.pointer_str = pointer_str
         self.no_pointer_str = " " * len(pointer_str)
         self.prompt_str = prompt_str
+        self.header_str = header_str
         self.output_stream = output_stream
-        self.keycodes_actions: Dict[str, str] = dict(ChainMap(*[{KEYS[v]: k for v in vlist} for k, vlist in ACTIONS.items()]))
+        self.keycodes_actions = get_keycodes_actions(actions)
         # Get the candidates
         if isinstance(candidates, Iterator) or callable(candidates):
             self.get_items_fn: Union[None, Callable[[], Sequence[Any]], Iterator[Any]] = candidates
@@ -130,9 +133,9 @@ class Finder:
                 self.process_key(get_char())
                 self.apply_filter()
                 self.update_screen()
-        except Confirm:
-            return self.prepare_result()
-        except Cancel:
+        except AcceptAction as accept:
+            return accept.selected_item
+        except AbortAction:
             return None
         finally:
             self.screen.cleanup()
@@ -140,7 +143,7 @@ class Finder:
     @property
     def screen_items(self) -> Sequence[Any]:
         "Candidates to be displayed on the screen"
-        return self.matching_candidates[self.offset : self.offset + self.max_candidates]
+        return self.matching_candidates[self.offset : self.offset + self.max_candidates_lines]
 
     @property
     def screen_items_len(self) -> int:
@@ -168,12 +171,17 @@ class Finder:
         return len(self.prompt_str.split(f"{NL}"))
 
     @property
-    def margin_lines(self) -> int:
-        "Screen margin"
-        return self.info_lines + self.prompt_lines
+    def header_lines(self) -> int:
+        "Number of header lines"
+        return len(self.header_str.split(f"{NL}")) if self.header_str else 0
 
     @property
-    def max_candidates(self) -> int:
+    def margin_lines(self) -> int:
+        "Screen margin"
+        return self.info_lines + self.prompt_lines + self.header_lines
+
+    @property
+    def max_candidates_lines(self) -> int:
         "Maximun number of candidates printables on the screen"
         return self.screen.height - self.margin_lines
 
@@ -184,7 +192,7 @@ class Finder:
         Args:
             input: initial search string
         """
-        self.input: str = input or ""
+        self.input = LineEditor(input or "")
         # Load the candidate list
         self.refresh_candidates()
         # Calculate the required height and setup the screen
@@ -209,24 +217,35 @@ class Finder:
         "Process the pressed key"
         action = self.keycodes_actions.get(ch)
         if action == "accept":  # Confirm
-            raise Confirm
+            raise AcceptAction(action, self.prepare_result(), ch)
         elif action == "abort":  # Cancel
-            raise Cancel
+            raise AbortAction(action, None, ch)
+        elif action == "custom":  # Custom action
+            raise CustomAction(action, self.prepare_result(), ch)
+        elif action == "backward-char":  # Move backward
+            self.input.backward_char()
+        elif action == "forward-char":  # Move forward
+            self.input.forward_char()
+        elif action == "beginning-of-line":  # Move to beginning of line
+            self.input.beginning_of_line()
+        elif action == "end-of-line":  # Move to end of line
+            self.input.end_of_line()
         elif action == "down":  # Move one line down
             self.selected = self.selected + 1
         elif action == "up":  # Move one line up
             self.selected = self.selected - 1
         elif action == "page-down":  # Move one page down
-            self.selected = self.selected + self.max_candidates
+            self.selected = self.selected + self.max_candidates_lines
         elif action == "page-up":  # Move one page up
-            self.selected = self.selected - self.max_candidates
+            self.selected = self.selected - self.max_candidates_lines
         elif action == "backward-delete-char":  # Delete one characted
-            if self.input:
-                self.input = self.input[:-1]
+            self.input.delete_backward_char()
+        elif action == "delete-char":  # Delete one characted
+            self.input.delete_char()
         elif action == "ignore":  # Skip
             pass
-        elif ch >= SPACE:  # Append the character to line
-            self.input = self.input + ch
+        elif ch >= SPACE:  # Add the character to line
+            self.input.insert(ch)
 
     def apply_filter(self) -> None:
         "Filter the items, calculate the screen offset"
@@ -234,8 +253,8 @@ class Finder:
         # Adject selected
         self.selected = max(min(self.selected, self.matching_candidates_len - 1), 0)
         # Calculate the offset
-        if self.selected >= self.offset + self.max_candidates:
-            self.offset = self.selected - self.max_candidates + 1
+        if self.selected >= self.offset + self.max_candidates_lines:
+            self.offset = self.selected - self.max_candidates_lines + 1
         elif self.selected < self.offset:
             self.offset = self.selected
         if self.offset < 0:
@@ -246,40 +265,46 @@ class Finder:
         if erase:
             self.screen.erase_screen()
         if self.layout == Layout.REVERSE_LIST:
+            self.print_header()
             self.print_items()
             self.print_empty_lines()
             self.print_info()
             self.print_prompt()
         self.screen.flush()
 
+    def print_header(self) -> None:
+        "Print header"
+        if self.header_str:
+            self.screen.erase_line().write(self.header_str).nl()
+
     def print_items(self) -> None:
+        "Print candidates"
         for i, item in enumerate(self.screen_items):
             is_selected = i + self.offset == self.selected
+            self.screen.erase_line()
             if is_selected:
-                self.screen.write(
-                    f"{ERASE_LINE}{RED}{BOLD}{BLACK_BG}{self.pointer_str}{NORMAL} {BOLD}{self.format_fn(item)}{NORMAL}{NL}"
-                )
+                self.screen.write(f"{RED}{BOLD}{BLACK_BG}{self.pointer_str} ").reset().bold()
             else:
-                self.screen.write(f"{ERASE_LINE}{BLACK_BG}{self.no_pointer_str}{NORMAL} {self.format_fn(item)}{NL}")
+                self.screen.write(f"{BLACK_BG}{self.no_pointer_str} ").reset()
+            self.screen.write(self.format_fn(item)).reset().nl()
 
     def print_empty_lines(self) -> None:
-        if self.fullscreen:
-            lines = self.max_candidates - self.screen_items_len
-        else:
-            lines = min(self.candidates_len, self.max_candidates - self.screen_items_len)
-        self.screen.write(f"{NL}" * lines)
+        "Print empty lines"
+        lines = self.max_candidates_lines - self.screen_items_len
+        self.screen.nl(lines)
 
     def print_info(self) -> None:
         "Print info"
         if self.info_style == InfoStyle.DEFAULT:
-            self.screen.write(f"  {ERASE_LINE}{YELLOW}{self.matching_candidates_len}/{self.candidates_len}{NORMAL}{NL}")
+            self.screen.erase_line().write(f"  {YELLOW}{self.matching_candidates_len}/{self.candidates_len}").reset().nl()
 
     def print_prompt(self) -> None:
         "Print prompt"
-        self.screen.write(f"{ERASE_LINE}{CYAN}{self.prompt_str}{NORMAL} {self.input}")
+        self.screen.erase_line().write(f"{CYAN}{self.prompt_str} ").reset()
+        self.input.print(self.screen)
 
     def match(self, item: Any) -> bool:
-        return self.input.lower() in self.format_fn(item).lower()
+        return str(self.input).lower() in self.format_fn(item).lower()
 
     def prepare_result(self) -> Any:
         "Output the selected item, if any"
